@@ -298,6 +298,9 @@ typedef struct
 	char		translation_dll[MEDIUM_REGISTRY_LEN];
 	char		translation_option[SMALL_REGISTRY_LEN];
 	char		focus_password;
+#ifdef	USE_LIBPQ
+	signed char	prefer_libpq;
+#endif /* USE_LIBPQ */
 	pgNAME		conn_settings;
 	signed char	disallow_premature;
 	signed char	allow_keyset;
@@ -319,6 +322,8 @@ typedef struct
 	signed char	disable_keepalive;
 	signed char	gssauth_use_gssapi;
 	UInt4		extra_opts;
+	Int4		keepalive_idle;
+	Int4		keepalive_interval;
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	signed char	xa_opt;
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
