@@ -193,11 +193,84 @@ abbrev_sslmode(const char *sslmode, char *abbrevmode)
 	return abbrevmode;
 }
 
+static char *
+makeKeepaliveConnectString(char *target, const ConnInfo *ci, BOOL abbrev)
+{
+	char	*buf = target;
+	*buf = '\0';
+
+	if (ci->disable_keepalive)
+		return target;
+
+	if (ci->keepalive_idle >= 0)
+	{
+		if (abbrev)
+			sprintf(buf, ABBR_KEEPALIVETIME "=%u;", ci->keepalive_idle);
+		else
+			sprintf(buf, INI_KEEPALIVETIME "=%u;", ci->keepalive_idle);
+		buf = strchr(buf, (int) '\0');
+	}
+	if (ci->keepalive_interval >= 0)
+	{
+		if (abbrev)
+			sprintf(buf, ABBR_KEEPALIVEINTERVAL "=%u;", ci->keepalive_interval);
+		else
+			sprintf(buf, INI_KEEPALIVEINTERVAL "=%u;", ci->keepalive_interval);
+	}
+	return target;
+}
+
+#ifdef	USE_LIBPQ
+static char *
+makePreferLibpqConnectString(char *target, const ConnInfo *ci, BOOL abbrev)
+{
+	char	*buf = target;
+	*buf = '\0';
+
+	if (ci->prefer_libpq < 0)
+		return target;
+
+	if (abbrev)
+		sprintf(buf, ABBR_PREFERLIBPQ "=%u;", ci->prefer_libpq);
+	else
+		sprintf(buf, INI_PREFERLIBPQ "=%u;", ci->prefer_libpq);
+	return target;
+}
+#endif /* USE_LIBPQ */
+
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+char *
+makeXaOptConnectString(char *target, const ConnInfo *ci, BOOL abbrev)
+{
+	char	*buf = target;
+	*buf = '\0';
+
+	if (ci->xa_opt < 0)
+		return target;
+
+	if (abbrev)
+	{
+		if (DEFAULT_XAOPT != ci->xa_opt)
+			sprintf(buf, ABBR_XAOPT "=%u;", ci->xa_opt);
+	}
+	else
+		sprintf(buf, INI_XAOPT "=%u;", ci->xa_opt);
+	return target;
+}
+#endif /* _HANDLE_ENLIST_IN_DTC_ */
+
 void
 makeConnectString(char *connect_string, const ConnInfo *ci, UWORD len)
 {
 	char		got_dsn = (ci->dsn[0] != '\0');
 	char		encoded_item[LARGE_REGISTRY_LEN];
+	char		keepaliveStr[32];
+#ifdef	USE_LIBPQ
+	char		preferLibpqStr[32];
+#endif
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+	char		xaOptStr[16];
+#endif
 	ssize_t		hlen, nlen, olen;
 	/*BOOL		abbrev = (len <= 400);*/
 	BOOL		abbrev = (len < 1024) || 0 < ci->force_abbrev_connstr;
@@ -268,6 +341,10 @@ inolog("hlen=%d", hlen);
 			INI_BYTEAASLONGVARBINARY "=%d;"
 			INI_USESERVERSIDEPREPARE "=%d;"
 			INI_LOWERCASEIDENTIFIER "=%d;"
+			"%s"
+#ifdef	USE_LIBPQ
+			"%s"
+#endif /* USE_LIBPQ */
 #ifdef	WIN32
 			INI_GSSAUTHUSEGSSAPI "=%d;"
 #endif /* WIN32 */
@@ -306,6 +383,10 @@ inolog("hlen=%d", hlen);
 			,ci->bytea_as_longvarbinary
 			,ci->use_server_side_prepare
 			,ci->lower_case_identifier
+			,makeKeepaliveConnectString(keepaliveStr, ci, FALSE)
+#ifdef	USE_LIBPQ
+			,makePreferLibpqConnectString(preferLibpqStr, ci, FALSE)
+#endif /* USE_LIBPQ */
 #ifdef	WIN32
 			,ci->gssauth_use_gssapi
 #endif /* WIN32 */
@@ -399,6 +480,13 @@ inolog("hlen=%d", hlen);
 				ABBR_MAXLONGVARCHARSIZE "=%d;"
 				INI_INT8AS "=%d;"
 				ABBR_EXTRASYSTABLEPREFIXES "=%s;"
+				"%s"
+#ifdef	USE_LIBPQ
+				"%s"
+#endif /* USE_LIBPQ */
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+				"%s"
+#endif /* _HANDLE_ENLIST_IN_DTC_ */
 				INI_ABBREVIATE "=%02x%x",
 				encoded_item,
 				ci->drivers.fetch_max,
@@ -407,6 +495,13 @@ inolog("hlen=%d", hlen);
 				ci->drivers.max_longvarchar_size,
 				ci->int8_as,
 				ci->drivers.extra_systable_prefixes,
+				makeKeepaliveConnectString(keepaliveStr, ci, TRUE),
+#ifdef	USE_LIBPQ
+				makePreferLibpqConnectString(preferLibpqStr, ci, TRUE),
+#endif /* USE_LIBPQ */
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+				makeXaOptConnectString(xaOptStr, ci, TRUE),
+#endif /* _HANDLE_ENLIST_IN_DTC_ */
 				EFFECTIVE_BIT_COUNT, flag);
 		if (olen < nlen && (PROTOCOL_74(ci) || ci->rollback_on_error >= 0))
 		{
@@ -602,6 +697,14 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		ci->lower_case_identifier = atoi(value);
 	else if (stricmp(attribute, INI_GSSAUTHUSEGSSAPI) == 0 || stricmp(attribute, ABBR_GSSAUTHUSEGSSAPI) == 0)
 		ci->gssauth_use_gssapi = atoi(value);
+	else if (stricmp(attribute, INI_KEEPALIVETIME) == 0 || stricmp(attribute, ABBR_KEEPALIVETIME) == 0)
+		ci->keepalive_idle = atoi(value);
+	else if (stricmp(attribute, INI_KEEPALIVEINTERVAL) == 0 || stricmp(attribute, ABBR_KEEPALIVEINTERVAL) == 0)
+		ci->keepalive_interval = atoi(value);
+#ifdef	USE_LIBPQ
+	else if (stricmp(attribute, INI_PREFERLIBPQ) == 0 || stricmp(attribute, ABBR_PREFERLIBPQ) == 0)
+		ci->prefer_libpq = atoi(value);
+#endif /* USE_LIBPQ */
 	else if (stricmp(attribute, INI_SSLMODE) == 0 || stricmp(attribute, ABBR_SSLMODE) == 0)
 	{
 		switch (value[0])
@@ -983,6 +1086,29 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 			ci->gssauth_use_gssapi = atoi(temp);
 	}
 
+	if (ci->keepalive_idle < 0 || overwrite)
+	{
+		SQLGetPrivateProfileString(DSN, INI_KEEPALIVETIME, "", temp, sizeof(temp), ODBC_INI);
+		if (temp[0])
+			if (0 == (ci->keepalive_idle = atoi(temp)))
+				ci->keepalive_idle = -1;
+	}
+	if (ci->keepalive_interval < 0 || overwrite)
+	{
+		SQLGetPrivateProfileString(DSN, INI_KEEPALIVEINTERVAL, "", temp, sizeof(temp), ODBC_INI);
+		if (temp[0])
+			if (0 == (ci->keepalive_interval = atoi(temp)))
+				ci->keepalive_interval = -1;
+	}
+#ifdef	USE_LIBPQ
+	if (ci->prefer_libpq < 0 || overwrite)
+	{
+		SQLGetPrivateProfileString(DSN, INI_PREFERLIBPQ, "-1", temp, sizeof(temp), ODBC_INI);
+		if (temp[0])
+			ci->prefer_libpq = atoi(temp);
+	}
+#endif /* USE_LIBPQ */
+
 	if (ci->sslmode[0] == '\0' || overwrite)
 		SQLGetPrivateProfileString(DSN, INI_SSLMODE, "", ci->sslmode, sizeof(ci->sslmode), ODBC_INI);
 
@@ -1274,6 +1400,23 @@ writeDSNinfo(const ConnInfo *ci)
 								 INI_SSLMODE,
 								 ci->sslmode,
 								 ODBC_INI);
+	sprintf(temp, "%d", ci->keepalive_idle);
+	SQLWritePrivateProfileString(DSN,
+								 INI_KEEPALIVETIME,
+								 temp,
+								 ODBC_INI);
+	sprintf(temp, "%d", ci->keepalive_interval);
+	SQLWritePrivateProfileString(DSN,
+								 INI_KEEPALIVEINTERVAL,
+								 temp,
+								 ODBC_INI);
+#ifdef	USE_LIBPQ
+	sprintf(temp, "%d", ci->prefer_libpq);
+	SQLWritePrivateProfileString(DSN,
+								 INI_PREFERLIBPQ,
+								 temp,
+								 ODBC_INI);
+#endif /* USE_LIBPQ */
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	sprintf(temp, "%d", ci->xa_opt);
 	SQLWritePrivateProfileString(DSN, INI_XAOPT, temp, ODBC_INI);
@@ -1611,136 +1754,13 @@ decode_or_remove_braces(const char *in)
 	return decode(in);
 }
 
-char *extract_attribute_setting(const char *str, const char *attr, BOOL ref_comment)
-{
-	const char *cptr, *sptr = NULL;
-	char	   *rptr;
-	BOOL	allowed_cmd = TRUE, in_quote = FALSE, in_comment = FALSE;
-	int	step = 0, skiplen;
-	size_t	len = 0, attrlen = strlen(attr);
-
-	for (cptr = str; *cptr; cptr++)
-	{
-		if (in_quote)
-		{
-			if (LITERAL_QUOTE == *cptr)
-			{
-				if (4 == step)
-				{
-					len = cptr - sptr;
-					step++;
-				}
-				in_quote = FALSE;
-			}
-			continue;
-		}
-		else if (in_comment)
-		{
-			if ('*' == *cptr &&
-			    '/' == cptr[1])
-			{
-				if (4 == step)
-				{
-					len = cptr - sptr;
-					step++;
-				}
-				in_comment = FALSE;
-				cptr++;
-				continue;
-			}
-			if (!ref_comment)
-				continue;
-		}
-		else if ('/' == *cptr &&
-			 '*' == cptr[1])
-		{
-			in_comment = TRUE;
-			cptr++;
-			continue;
-		}
-		if (';' == *cptr)
-		{
-			if (4 == step)
-			{
-				len = cptr - sptr;
-			}
-			allowed_cmd = TRUE;
-			step = 0;
-			continue;
-		}
-		if (!allowed_cmd)
-			continue;
-		if (isspace((unsigned char) *cptr))
-		{
-			if (4 == step)
-			{
-				len =  cptr - sptr;
-				step++;
-			}
-			continue;
-		}
-		switch (step)
-		{
-			case 0:
-				if (0 != strnicmp(cptr, "set", 3))
-				{
-					allowed_cmd = FALSE;
-					continue;
-				}
-				step++;
-				cptr += 3;
-				break;
-			case 1:
-				if (0 != strnicmp(cptr, attr, attrlen))
-				{
-					allowed_cmd = FALSE;
-					continue;
-				}
-				step++;
-				cptr += (attrlen - 1);
-				break;
-			case 2:
-				skiplen = 0;
-				if (0 != strnicmp(cptr, "=", 1))
-				{
-					skiplen = (int) strlen("to");
-					if (0 != strnicmp(cptr, "to", 2))
-					{
-						allowed_cmd = FALSE;
-						continue;
-					}
-				}
-				step++;
-				cptr += skiplen;
-				break;
-			case 3:
-				if (LITERAL_QUOTE == *cptr)
-				{
-					cptr++;
-					sptr = cptr;
-				}
-				else
-					sptr = cptr;
-				step++;
-				break;
-		}
-	}
-	if (!sptr)
-		return NULL;
-	rptr = malloc(len + 1);
-	memcpy(rptr, sptr, len);
-	rptr[len] = '\0';
-	mylog("extracted a %s '%s' from %s\n", attr, rptr, str);
-	return rptr;
-}
-
 /*
  *	extract the specified attribute from the comment part.
  *		attribute=[']value[']
  */
 char *extract_extra_attribute_setting(const pgNAME setting, const char *attr)
 {
-	const char *str = GET_NAME(setting);
+	const char *str = SAFE_NAME(setting);
 	const char *cptr, *sptr = NULL;
 	char	   *rptr;
 	BOOL	allowed_cmd = FALSE, in_quote = FALSE, in_comment = FALSE;
