@@ -147,6 +147,14 @@ extern "C" {
 #define INI_LOGDIR			"Logdir"
 #define INI_GSSAUTHUSEGSSAPI		"GssAuthUseGSS"
 #define ABBR_GSSAUTHUSEGSSAPI		"D0"
+#define INI_KEEPALIVETIME		"KeepaliveTime"
+#define ABBR_KEEPALIVETIME		"D1"
+#define INI_KEEPALIVEINTERVAL		"KeepaliveInterval"
+#define ABBR_KEEPALIVEINTERVAL		"D2"
+#define INI_DTCLOG			"Dtclog"
+#define INI_PREFERLIBPQ			"PreferLibpq"
+#define ABBR_PREFERLIBPQ		"D3"
+#define ABBR_XAOPT			"D4"
 
 #define	SSLMODE_DISABLE		"disable"
 #define	SSLMODE_ALLOW		"allow"
@@ -162,7 +170,6 @@ extern "C" {
 
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 #define INI_XAOPT			"XaOpt"
-const char *GetXaLibPath(void);
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 /* Bit representaion for abbreviated connection strings */
 #define BIT_LFCONVERSION			(1L)
@@ -275,6 +282,10 @@ LRESULT CALLBACK ds_options2Proc(HWND hdlg,
 			   UINT wMsg,
 			   WPARAM wParam,
 			   LPARAM lParam);
+LRESULT CALLBACK ds_options3Proc(HWND hdlg,
+			   UINT wMsg,
+			   WPARAM wParam,
+			   LPARAM lParam);
 LRESULT CALLBACK manage_dsnProc(HWND hdlg,
 			   UINT wMsg,
 			   WPARAM wParam,
@@ -296,7 +307,6 @@ int	setLogDir(const char *dir);
 int     changeDriverNameOfaDSN(const char *dsn, const char *driver_name, DWORD *errcode);
 UInt4	getExtraOptions(const ConnInfo *);
 BOOL	setExtraOptions(ConnInfo *, const char *str, const char *format);
-char	*extract_attribute_setting(const char *str, const char *attr, BOOL ref_comment);
 char	*extract_extra_attribute_setting(const pgNAME setting, const char *attr);
 
 #ifdef	__cplusplus
