@@ -27,6 +27,10 @@
 
 #include "multibyte.h"
 #include "pgapifunc.h"
+#ifdef _HANDLE_ENLIST_IN_DTC_
+#include "xalibname.h"
+#include "connexp.h"
+#endif /* _HANDLE_ENLIST_IN_DTC_ */
 
 extern GLOBAL_VALUES globals;
 
@@ -34,6 +38,11 @@ extern HINSTANCE NEAR s_hModule;
 static int	driver_optionsDraw(HWND, const ConnInfo *, int src, BOOL enable);
 static int	driver_options_update(HWND hdlg, ConnInfo *ci, const char *);
 
+static int	ds_options_update(HWND hdlg, ConnInfo *ci);
+
+static int	ds_options3Draw(HWND, const ConnInfo *);
+static int	ds_options3_update(HWND hdlg, ConnInfo *ci);
+
 static struct {
 	int	ids;
 	const char * const	modestr;
@@ -136,7 +145,7 @@ GetDlgStuff(HWND hdlg, ConnInfo *ci)
 static int
 driver_optionsDraw(HWND hdlg, const ConnInfo *ci, int src, BOOL enable)
 {
-	const GLOBAL_VALUES *comval;
+	const GLOBAL_VALUES *comval = NULL;
 	static BOOL defset = FALSE;
 	static GLOBAL_VALUES defval;
 
@@ -171,6 +180,8 @@ driver_optionsDraw(HWND hdlg, const ConnInfo *ci, int src, BOOL enable)
 			defset = TRUE;
 			comval = &defval;
 			break;
+		default:
+			return 0;
 	}
 
 	ShowWindow(GetDlgItem(hdlg, DRV_MSG_LABEL2), enable ? SW_SHOW : SW_HIDE);
@@ -222,8 +233,8 @@ driver_optionsDraw(HWND hdlg, const ConnInfo *ci, int src, BOOL enable)
 	/* Driver Connection Settings */
 	SetDlgItemText(hdlg, DRV_CONNSETTINGS, SAFE_NAME(comval->conn_settings));
 	EnableWindow(GetDlgItem(hdlg, DRV_CONNSETTINGS), enable);
-	ShowWindow(GetDlgItem(hdlg, IDPREVPAGE), enable ? SW_HIDE : SW_SHOW);
-	ShowWindow(GetDlgItem(hdlg, IDNEXTPAGE), enable ? SW_HIDE : SW_SHOW);
+	ShowWindow(GetDlgItem(hdlg, ID2NDPAGE), enable ? SW_HIDE : SW_SHOW);
+	ShowWindow(GetDlgItem(hdlg, ID3RDPAGE), enable ? SW_HIDE : SW_SHOW);
 	return 0;
 }
 
@@ -363,11 +374,13 @@ global_optionsProc(HWND hdlg,
 	HMODULE	hmodule;
 	FARPROC	proc;
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
+	ConnInfo	*ci;
 	char logdir[PATH_MAX];
 
 	switch (wMsg)
 	{
 		case WM_INITDIALOG:
+			SetWindowLongPtr(hdlg, DWLP_USER, lParam); /* save for test etc */ 
 			CheckDlgButton(hdlg, DRV_COMMLOG, globals.commlog);
 #ifndef Q_LOG
 			EnableWindow(GetDlgItem(hdlg, DRV_COMMLOG), FALSE);
@@ -395,6 +408,7 @@ global_optionsProc(HWND hdlg,
 			break;
 
 		case WM_COMMAND:
+			ci = (ConnInfo *) GetWindowLongPtr(hdlg, DWLP_USER);
 			switch (GET_WM_COMMAND_ID(wParam, lParam))
 			{
 				case IDOK:
@@ -445,7 +459,7 @@ ds_options1Proc(HWND hdlg,
 						fbuf,
 						sizeof(fbuf));
 				if (cmd <= 0)
-					strcpy(fbuf, "Advanced Options (%s) 1/2");
+					strcpy(fbuf, "Advanced Options (%s) 1/3");
 				sprintf(strbuf, fbuf, ci->dsn);
 				SetWindowText(hdlg, strbuf);
 			}
@@ -478,20 +492,121 @@ ds_options1Proc(HWND hdlg,
 					driver_optionsDraw(hdlg, ci, 0, FALSE);
 					break;
 
-				case IDNEXTPAGE:
+				case ID2NDPAGE:
 					driver_options_update(hdlg, ci, NULL);
-
 					EndDialog(hdlg, FALSE);
 					DialogBoxParam(s_hModule,
 								   MAKEINTRESOURCE(DLG_OPTIONS_DS),
 								   hdlg, ds_options2Proc, (LPARAM) ci);
 					break;
+				case ID3RDPAGE:
+					driver_options_update(hdlg, ci, NULL);
+					EndDialog(hdlg, FALSE);
+					DialogBoxParam(s_hModule,
+								   MAKEINTRESOURCE(DLG_OPTIONS_DS3),
+								   hdlg, ds_options3Proc, (LPARAM) ci);
+					break;
 			}
 	}
 
 	return FALSE;
 }
 
+static int
+ds_options_update(HWND hdlg, ConnInfo *ci)
+{
+	char		buf[128];
+
+	mylog("%s: got ci = %p\n", __FUNCTION__, ci);
+
+	/* Readonly */
+	sprintf(ci->onlyread, "%d", IsDlgButtonChecked(hdlg, DS_READONLY));
+
+	/* Protocol */
+	if (IsDlgButtonChecked(hdlg, DS_PG62))
+		strcpy(ci->protocol, PG62);
+	else if (IsDlgButtonChecked(hdlg, DS_PG63))
+		strcpy(ci->protocol, PG63);
+	else if (IsDlgButtonChecked(hdlg, DS_PG64))
+		strcpy(ci->protocol, PG64);
+	else
+		/* latest */
+		strcpy(ci->protocol, PG74);
+
+	/* Issue rollback command on error */
+	if (IsDlgButtonChecked(hdlg, DS_NO_ROLLBACK))
+		ci->rollback_on_error = 0;
+	else if (IsDlgButtonChecked(hdlg, DS_TRANSACTION_ROLLBACK))
+		ci->rollback_on_error = 1;
+	else if (IsDlgButtonChecked(hdlg, DS_STATEMENT_ROLLBACK))
+		ci->rollback_on_error = 2;
+	else
+		/* legacy */
+		ci->rollback_on_error = 1;
+
+	/* Int8 As */
+	if (IsDlgButtonChecked(hdlg, DS_INT8_AS_DEFAULT))
+		ci->int8_as = 0;
+	else if (IsDlgButtonChecked(hdlg, DS_INT8_AS_BIGINT))
+		ci->int8_as = SQL_BIGINT;
+	else if (IsDlgButtonChecked(hdlg, DS_INT8_AS_NUMERIC))
+		ci->int8_as = SQL_NUMERIC;
+	else if (IsDlgButtonChecked(hdlg, DS_INT8_AS_DOUBLE))
+		ci->int8_as = SQL_DOUBLE;
+	else if (IsDlgButtonChecked(hdlg, DS_INT8_AS_INT4))
+		ci->int8_as = SQL_INTEGER;
+	else
+		ci->int8_as = SQL_VARCHAR;
+
+	GetDlgItemText(hdlg, DS_EXTRA_OPTIONS, buf, sizeof(buf));
+	setExtraOptions(ci, buf, NULL);
+	sprintf(ci->show_system_tables, "%d", IsDlgButtonChecked(hdlg, DS_SHOWSYSTEMTABLES));
+
+	sprintf(ci->row_versioning, "%d", IsDlgButtonChecked(hdlg, DS_ROWVERSIONING));
+	ci->disallow_premature = IsDlgButtonChecked(hdlg, DS_DISALLOWPREMATURE);
+	ci->lf_conversion = IsDlgButtonChecked(hdlg, DS_LFCONVERSION);
+	ci->true_is_minus1 = IsDlgButtonChecked(hdlg, DS_TRUEISMINUS1);
+	ci->allow_keyset = IsDlgButtonChecked(hdlg, DS_UPDATABLECURSORS);
+	ci->use_server_side_prepare = IsDlgButtonChecked(hdlg, DS_SERVERSIDEPREPARE);
+	ci->bytea_as_longvarbinary = IsDlgButtonChecked(hdlg, DS_BYTEAASLONGVARBINARY);
+	/*ci->lower_case_identifier = IsDlgButtonChecked(hdlg, DS_LOWERCASEIDENTIFIER);*/
+	ci->gssauth_use_gssapi = IsDlgButtonChecked(hdlg, DS_GSSAUTHUSEGSSAPI);
+
+	/* OID Options */
+	sprintf(ci->fake_oid_index, "%d", IsDlgButtonChecked(hdlg, DS_FAKEOIDINDEX));
+	sprintf(ci->show_oid_column, "%d", IsDlgButtonChecked(hdlg, DS_SHOWOIDCOLUMN));
+
+	/* Datasource Connection Settings */
+	{
+		char conn_settings[LARGE_REGISTRY_LEN];
+		GetDlgItemText(hdlg, DS_CONNSETTINGS, conn_settings, sizeof(conn_settings));
+		if ('\0' != conn_settings[0])
+			STR_TO_NAME(ci->conn_settings, conn_settings);
+	}
+
+	/* TCP KEEPALIVE */
+	ci->disable_keepalive = IsDlgButtonChecked(hdlg, DS_DISABLE_KEEPALIVE);
+	if (ci->disable_keepalive)
+	{
+		ci->keepalive_idle = -1;
+		ci->keepalive_interval = -1;
+	}
+	else
+	{
+		char	temp[64];
+		int	val;
+
+		GetDlgItemText(hdlg, DS_KEEPALIVETIME, temp, sizeof(temp));
+		if  (val = atoi(temp), 0 == val)
+			val = -1;
+		ci->keepalive_idle = val;
+		GetDlgItemText(hdlg, DS_KEEPALIVEINTERVAL, temp, sizeof(temp));
+		if  (val = atoi(temp), 0 == val)
+			val = -1;
+		ci->keepalive_interval = val;
+	}
+	return 0;
+}
 
 LRESULT			CALLBACK
 ds_options2Proc(HWND hdlg,
@@ -520,7 +635,7 @@ ds_options2Proc(HWND hdlg,
 						fbuf,
 						sizeof(fbuf));
 				if (cmd <= 0)
-					strcpy(fbuf, "Advanced Options (%s) 2/2");
+					strcpy(fbuf, "Advanced Options (%s) 2/3");
 				sprintf(buf, fbuf, ci->dsn);
 				SetWindowText(hdlg, buf);
 			}
@@ -607,109 +722,224 @@ ds_options2Proc(HWND hdlg,
 
 			/* Datasource Connection Settings */
 			SetDlgItemText(hdlg, DS_CONNSETTINGS, SAFE_NAME(ci->conn_settings));
+			/* KEEPALIVE */
+			enable = (0 == (ci->extra_opts & BIT_DISABLE_KEEPALIVE));
+			CheckDlgButton(hdlg, DS_DISABLE_KEEPALIVE, !enable);
+			if (enable)
+			{
+				if (ci->keepalive_idle > 0)
+				{
+					snprintf(buf, sizeof(buf), "%d", ci->keepalive_idle);
+					SetDlgItemText(hdlg, DS_KEEPALIVETIME, buf);
+				}
+				if (ci->keepalive_interval > 0)
+				{
+					snprintf(buf, sizeof(buf), "%d", ci->keepalive_interval);
+					SetDlgItemText(hdlg, DS_KEEPALIVEINTERVAL, buf);
+				}
+			}
 			break;
 
 		case WM_COMMAND:
+			ci = (ConnInfo *) GetWindowLongPtr(hdlg, DWLP_USER);
 			switch (cmd = GET_WM_COMMAND_ID(wParam, lParam))
 			{
 				case DS_SHOWOIDCOLUMN:
 					mylog("WM_COMMAND: DS_SHOWOIDCOLUMN\n");
 					EnableWindow(GetDlgItem(hdlg, DS_FAKEOIDINDEX), IsDlgButtonChecked(hdlg, DS_SHOWOIDCOLUMN));
 					return TRUE;
+				case DS_DISABLE_KEEPALIVE:
+					mylog("WM_COMMAND: DS_SHOWOIDCOLUMN\n");
+					EnableWindow(GetDlgItem(hdlg, DS_KEEPALIVETIME), !IsDlgButtonChecked(hdlg, cmd));
+					EnableWindow(GetDlgItem(hdlg, DS_KEEPALIVEINTERVAL), !IsDlgButtonChecked(hdlg, cmd));
+					return TRUE;
 
 				case IDOK:
-				case IDAPPLY:
-				case IDPREVPAGE:
-					ci = (ConnInfo *) GetWindowLongPtr(hdlg, DWLP_USER);
-					mylog("IDOK: got ci = %p\n", ci);
-
-					/* Readonly */
-					sprintf(ci->onlyread, "%d", IsDlgButtonChecked(hdlg, DS_READONLY));
-
-					/* Protocol */
-					if (IsDlgButtonChecked(hdlg, DS_PG62))
-						strcpy(ci->protocol, PG62);
-					else if (IsDlgButtonChecked(hdlg, DS_PG63))
-						strcpy(ci->protocol, PG63);
-					else if (IsDlgButtonChecked(hdlg, DS_PG64))
-						strcpy(ci->protocol, PG64);
-					else
-						/* latest */
-						strcpy(ci->protocol, PG74);
-
-					/* Issue rollback command on error */
-					if (IsDlgButtonChecked(hdlg, DS_NO_ROLLBACK))
-						ci->rollback_on_error = 0;
-					else if (IsDlgButtonChecked(hdlg, DS_TRANSACTION_ROLLBACK))
-						ci->rollback_on_error = 1;
-					else if (IsDlgButtonChecked(hdlg, DS_STATEMENT_ROLLBACK))
-						ci->rollback_on_error = 2;
-					else
-						/* legacy */
-						ci->rollback_on_error = 1;
-
-					/* Int8 As */
-					if (IsDlgButtonChecked(hdlg, DS_INT8_AS_DEFAULT))
-						ci->int8_as = 0;
-					else if (IsDlgButtonChecked(hdlg, DS_INT8_AS_BIGINT))
-						ci->int8_as = SQL_BIGINT;
-					else if (IsDlgButtonChecked(hdlg, DS_INT8_AS_NUMERIC))
-						ci->int8_as = SQL_NUMERIC;
-					else if (IsDlgButtonChecked(hdlg, DS_INT8_AS_DOUBLE))
-						ci->int8_as = SQL_DOUBLE;
-					else if (IsDlgButtonChecked(hdlg, DS_INT8_AS_INT4))
-						ci->int8_as = SQL_INTEGER;
-					else
-						ci->int8_as = SQL_VARCHAR;
-
-					GetDlgItemText(hdlg, DS_EXTRA_OPTIONS, buf, sizeof(buf));
-					setExtraOptions(ci, buf, NULL);
-					sprintf(ci->show_system_tables, "%d", IsDlgButtonChecked(hdlg, DS_SHOWSYSTEMTABLES));
-
-					sprintf(ci->row_versioning, "%d", IsDlgButtonChecked(hdlg, DS_ROWVERSIONING));
-					ci->disallow_premature = IsDlgButtonChecked(hdlg, DS_DISALLOWPREMATURE);
-					ci->lf_conversion = IsDlgButtonChecked(hdlg, DS_LFCONVERSION);
-					ci->true_is_minus1 = IsDlgButtonChecked(hdlg, DS_TRUEISMINUS1);
-					ci->allow_keyset = IsDlgButtonChecked(hdlg, DS_UPDATABLECURSORS);
-					ci->use_server_side_prepare = IsDlgButtonChecked(hdlg, DS_SERVERSIDEPREPARE);
-					ci->bytea_as_longvarbinary = IsDlgButtonChecked(hdlg, DS_BYTEAASLONGVARBINARY);
-					/*ci->lower_case_identifier = IsDlgButtonChecked(hdlg, DS_LOWERCASEIDENTIFIER);*/
-					ci->gssauth_use_gssapi = IsDlgButtonChecked(hdlg, DS_GSSAUTHUSEGSSAPI);
-
-					/* OID Options */
-					sprintf(ci->fake_oid_index, "%d", IsDlgButtonChecked(hdlg, DS_FAKEOIDINDEX));
-					sprintf(ci->show_oid_column, "%d", IsDlgButtonChecked(hdlg, DS_SHOWOIDCOLUMN));
-
-					/* Datasource Connection Settings */
-					{
-						char conn_settings[LARGE_REGISTRY_LEN];
-						GetDlgItemText(hdlg, DS_CONNSETTINGS, conn_settings, sizeof(conn_settings));
-						if ('\0' != conn_settings[0])
-							STR_TO_NAME(ci->conn_settings, conn_settings);
-					}
-					if (IDAPPLY == cmd)
-					{
-						SendMessage(GetWindow(hdlg, GW_OWNER), WM_COMMAND, wParam, lParam);
-						break;
-					}
-
-					EndDialog(hdlg, cmd == IDOK);
-					if (IDOK == cmd)
-						return TRUE;
-					DialogBoxParam(s_hModule,
-								   MAKEINTRESOURCE(DLG_OPTIONS_DRV),
-								   hdlg, ds_options1Proc, (LPARAM) ci);
-					break;
-
+					ds_options_update(hdlg, ci);
 				case IDCANCEL:
-					EndDialog(hdlg, GET_WM_COMMAND_ID(wParam, lParam) == IDOK);
+					EndDialog(hdlg, IDOK == cmd);
 					return TRUE;
+				case IDAPPLY:
+					ds_options_update(hdlg, ci);
+					SendMessage(GetWindow(hdlg, GW_OWNER), WM_COMMAND, wParam, lParam);
+					break;
+				case ID1STPAGE:
+					ds_options_update(hdlg, ci);
+					EndDialog(hdlg, cmd == IDOK);
+					DialogBoxParam(s_hModule,
+						MAKEINTRESOURCE(DLG_OPTIONS_DRV),
+						   hdlg, ds_options1Proc, (LPARAM) ci);
+					break;
+				case ID3RDPAGE:
+					ds_options_update(hdlg, ci);
+					EndDialog(hdlg, cmd == IDOK);
+					DialogBoxParam(s_hModule,
+						MAKEINTRESOURCE(DLG_OPTIONS_DS3),
+						   hdlg, ds_options3Proc, (LPARAM) ci);
+					break;
 			}
 	}
 
 	return FALSE;
 }
 
+static int
+ds_options3Draw(HWND hdlg, const ConnInfo *ci)
+{
+	BOOL	enable = TRUE;
+	static BOOL defset = FALSE;
+
+	/* The use of LIBPQ library */
+#ifdef	USE_LIBPQ
+	if (ci->prefer_libpq < 0)
+		CheckDlgButton(hdlg, DS_DEFAULT_LIBPQ_USE, 1);
+	else if (0 == ci->prefer_libpq)
+		CheckDlgButton(hdlg, DS_NO_LIBPQ_USE, 1);
+	else 
+		CheckDlgButton(hdlg, DS_LIBPQ_USE, 1);
+#else
+	enable = FALSE;
+	EnableWindow(GetDlgItem(hdlg, DS_DEFAULT_LIBPQ_USE), enable);
+	EnableWindow(GetDlgItem(hdlg, DS_LIBPQ_USE), enable);
+	EnableWindow(GetDlgItem(hdlg, DS_NO_LIBPQ_USE), enable);
+#endif /* USE_LIBPQ */
+
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+	switch (ci->xa_opt)
+	{
+		case 0:
+			enable = FALSE;
+			break;
+		case DTC_CHECK_LINK_ONLY:
+			CheckDlgButton(hdlg, DS_DTC_LINK_ONLY, 1);
+			break;
+		case DTC_CHECK_BEFORE_LINK:
+			CheckDlgButton(hdlg, DS_DTC_SIMPLE_PRECHECK, 1);
+			break;
+		case DTC_CHECK_RM_CONNECTION:
+			CheckDlgButton(hdlg, DS_DTC_CONFIRM_RM_CONNECTION, 1);
+			break;
+	}
+#else
+	enable = FALSE;
+#endif /* _HANDLE_ENLIST_IN_DTC_ */
+	if (!enable)
+	{
+		EnableWindow(GetDlgItem(hdlg, DS_DTC_LINK_ONLY), enable);
+		EnableWindow(GetDlgItem(hdlg, DS_DTC_SIMPLE_PRECHECK), enable);
+		EnableWindow(GetDlgItem(hdlg, DS_DTC_CONFIRM_RM_CONNECTION), enable);
+	}
+
+	return 0;
+}
+
+static int
+ds_options3_update(HWND hdlg, ConnInfo *ci)
+{
+	mylog("%s: got ci = %p\n", __FUNCTION__, ci);
+
+#ifdef	USE_LIBPQ
+	/* Libpq use */
+	if (IsDlgButtonChecked(hdlg, DS_DEFAULT_LIBPQ_USE))
+		ci->prefer_libpq = -1;
+	else if (IsDlgButtonChecked(hdlg, DS_NO_LIBPQ_USE))
+		ci->prefer_libpq = 0;
+	else
+		ci->prefer_libpq = 1;
+#endif /* USE_LIBPQ */
+
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+	if (IsDlgButtonChecked(hdlg, DS_DTC_LINK_ONLY))
+		ci->xa_opt = DTC_CHECK_LINK_ONLY;
+	else if (IsDlgButtonChecked(hdlg, DS_DTC_SIMPLE_PRECHECK))
+		ci->xa_opt = DTC_CHECK_BEFORE_LINK;
+	else if (IsDlgButtonChecked(hdlg, DS_DTC_CONFIRM_RM_CONNECTION))
+		ci->xa_opt = DTC_CHECK_RM_CONNECTION;
+	else
+		ci->xa_opt = 0;
+#endif /* _HANDLE_ENLIST_IN_DTC_ */
+
+	return 0;
+}
+
+LRESULT			CALLBACK
+ds_options3Proc(HWND hdlg,
+			   UINT wMsg,
+			   WPARAM wParam,
+			   LPARAM lParam)
+{
+	ConnInfo   *ci, tmpInfo;
+	char		buf[128];
+	DWORD		cmd;
+
+	switch (wMsg)
+	{
+		case WM_INITDIALOG:
+			ci = (ConnInfo *) lParam;
+			SetWindowLongPtr(hdlg, DWLP_USER, lParam);		/* save for OK */
+
+			/* Change window caption */
+			if (ci && ci->dsn && ci->dsn[0])
+			{
+				char	fbuf[64];
+
+				cmd = LoadString(s_hModule,
+						IDS_ADVANCE_OPTION_DSN3,
+						fbuf,
+						sizeof(fbuf));
+				if (cmd <= 0)
+					strcpy(fbuf, "Advanced Options (%s) 3/3");
+				sprintf(buf, fbuf, ci->dsn);
+				SetWindowText(hdlg, buf);
+			}
+			else
+			{
+				LoadString(s_hModule, IDS_ADVANCE_OPTION_CON3, buf, sizeof(buf));
+				SetWindowText(hdlg, buf);
+				ShowWindow(GetDlgItem(hdlg, IDAPPLY), SW_HIDE);				}
+
+			ds_options3Draw(hdlg, ci);
+			break;
+
+		case WM_COMMAND:
+			ci = (ConnInfo *) GetWindowLongPtr(hdlg, DWLP_USER);
+			switch (cmd = GET_WM_COMMAND_ID(wParam, lParam))
+			{
+				case IDOK:
+					ds_options3_update(hdlg, ci);
+				case IDCANCEL:
+					EndDialog(hdlg, IDOK == cmd);
+					return TRUE;
+				case IDAPPLY:
+					ds_options3_update(hdlg, ci);
+					SendMessage(GetWindow(hdlg, GW_OWNER), WM_COMMAND, wParam, lParam);
+					break;
+				case IDC_TEST:
+					memcpy(&tmpInfo, ci, sizeof(tmpInfo));
+					ds_options3_update(hdlg, &tmpInfo);
+					test_connection(hdlg, &tmpInfo, TRUE);
+					break;
+				case ID1STPAGE:
+					ds_options3_update(hdlg, ci);
+					EndDialog(hdlg, cmd == IDOK);
+					DialogBoxParam(s_hModule,
+						MAKEINTRESOURCE(DLG_OPTIONS_DRV),
+						   hdlg, ds_options1Proc, (LPARAM) ci);
+					break;
+				case ID2NDPAGE:
+					ds_options3_update(hdlg, ci);
+					EndDialog(hdlg, cmd == IDOK);
+					DialogBoxParam(s_hModule,
+						MAKEINTRESOURCE(DLG_OPTIONS_DS),
+						   hdlg, ds_options2Proc, (LPARAM) ci);
+					break;
+			}
+	}
+
+	return FALSE;
+}
+
+
 typedef	SQLRETURN (SQL_API *SQLAPIPROC)();
 static int
 makeDriversList(HWND lwnd, const ConnInfo *ci)
