@@ -51,12 +51,6 @@ PGAPI_Prepare(HSTMT hstmt,
 
 #define	return	DONT_CALL_RETURN_FROM_HERE???
 	/* StartRollbackState(self); */
-	if (!self)
-	{
-		SC_log_error(func, "", NULL);
-		retval = SQL_INVALID_HANDLE;
-		goto cleanup;
-	}
 
 	/*
 	 * According to the ODBC specs it is valid to call SQLPrepare multiple
@@ -643,7 +637,7 @@ int
 StartRollbackState(StatementClass *stmt)
 {
 	CSTR	func = "StartRollbackState";
-	int	ret;
+	int			ret;
 	ConnectionClass	*conn;
 	ConnInfo	*ci = NULL;
 
@@ -651,7 +645,7 @@ inolog("%s:%p->internal=%d\n", func, stmt, stmt->internal);
 	conn = SC_get_conn(stmt);
 	if (conn)
 		ci = &conn->connInfo;
-	ret = 0;
+
 	if (!ci || ci->rollback_on_error < 0) /* default */
 	{
 		if (conn && PG_VERSION_GE(conn, 8.0))
@@ -665,6 +659,7 @@ inolog("%s:%p->internal=%d\n", func, stmt, stmt->internal);
 		if (2 == ret && PG_VERSION_LT(conn, 8.0))
 			ret = 1;
 	}
+
 	switch (ret)
 	{
 		case 1:
@@ -868,45 +863,38 @@ SC_setInsertedTable(StatementClass *stmt, RETCODE retval)
 	NULL_THE_NAME(conn->tableIns);
 	if (!SQL_SUCCEEDED(retval))
 		return;
-	ptr = NULL;
-	if (IDENTIFIER_QUOTE == *cmd)
+	while (TRUE)
 	{
-		if (ptr = strchr(cmd + 1, IDENTIFIER_QUOTE), NULL == ptr)
-			return;
-		if ('.' == ptr[1])
+		if (IDENTIFIER_QUOTE == *cmd)
 		{
+			if (ptr = strchr(cmd + 1, IDENTIFIER_QUOTE), NULL == ptr)	/* syntax error */
+			{
+				NULL_THE_NAME(conn->schemaIns);
+				NULL_THE_NAME(conn->tableIns);
+				break;
+			}
 			len = ptr - cmd - 1;
-			STRN_TO_NAME(conn->schemaIns, cmd + 1, len);
-			cmd = ptr + 2;
-			ptr = NULL;
+			cmd++;
+			ptr++;
 		}
-	}
-	else
-	{
-		if (ptr = strchr(cmd + 1, '.'), NULL != ptr)
+		else
 		{
-			len = ptr - cmd;
-			STRN_TO_NAME(conn->schemaIns, cmd, len);
-			cmd = ptr + 1;
-			ptr = NULL;
+			if (ptr = strchr(cmd + 1, '.'), NULL != ptr)
+				len = ptr - cmd;
+			else
+			{
+				ptr = cmd;
+				while (*ptr && !isspace((UCHAR) *ptr)) ptr++;
+				len = ptr - cmd;
+			}
 		}
-	}
-	if (IDENTIFIER_QUOTE == *cmd && NULL == ptr)
-	{
-		if (ptr = strchr(cmd + 1, IDENTIFIER_QUOTE), NULL == ptr)
-			return;
-	}
-	if (IDENTIFIER_QUOTE == *cmd)
-	{
-		len = ptr - cmd - 1;
-		STRN_TO_NAME(conn->tableIns, cmd + 1, len);
-	}
-	else
-	{
-		ptr = cmd;
-		while (*ptr && !isspace((UCHAR) *ptr)) ptr++;
-		len = ptr - cmd;
+		if (NAME_IS_VALID(conn->tableIns))
+			MOVE_NAME(conn->schemaIns, conn->tableIns);
 		STRN_TO_NAME(conn->tableIns, cmd, len);
+		if ('.' == *ptr)
+			cmd = ptr + 1;
+		else
+			break;
 	}
 }
 
@@ -926,13 +914,6 @@ PGAPI_Execute(HSTMT hstmt, UWORD flag)
 
 	mylog("%s: entering...%x\n", func, flag);
 
-	if (!stmt)
-	{
-		SC_log_error(func, "", NULL);
-		mylog("%s: NULL statement so return SQL_INVALID_HANDLE\n", func);
-		return SQL_INVALID_HANDLE;
-	}
-
 	conn = SC_get_conn(stmt);
 	apdopts = SC_get_APDF(stmt);
 	/*
@@ -988,16 +969,13 @@ PGAPI_Execute(HSTMT hstmt, UWORD flag)
 	else if (PREPARED_PERMANENTLY == stmt->prepared ||
 		 PREPARED_TEMPORARILY == stmt->prepared)
 	{
-		QResultClass	*res;
-
 		/*
 		 * re-executing an prepared statement.
 		 * Don't recycle the statement but
 		 * discard the old result.
 		 */
 		recycle = FALSE;
-		if (res = SC_get_Result(stmt), res)
-			QR_close_result(res, FALSE);
+		SC_reset_result_for_rerun(stmt);
 	}
 	/*
 	 * If SQLExecute is being called again, recycle the statement. Note
@@ -1246,7 +1224,6 @@ PGAPI_Cancel(HSTMT hstmt)		/* Statement to cancel. */
 	StatementClass *stmt = (StatementClass *) hstmt, *estmt;
 	ConnectionClass *conn;
 	RETCODE		ret = SQL_SUCCESS;
-	BOOL	entered_cs = FALSE;
 
 	mylog("%s: entering...\n", func);
 
@@ -1258,92 +1235,108 @@ PGAPI_Cancel(HSTMT hstmt)		/* Statement to cancel. */
 	}
 	conn = SC_get_conn(stmt);
 
-#define	return	DONT_CALL_RETURN_FROM_HERE???
-	/* StartRollbackState(stmt); */
-
 	if (stmt->execute_delegate)
 		estmt = stmt->execute_delegate;
 	else
 		estmt = stmt;
+
 	/*
-	 * Not in the middle of SQLParamData/SQLPutData so cancel like a
-	 * close.
+	 * SQLCancel works differently depending on what the statement is
+	 * currently doing:
+	 *
+	 * 1. In the middle of SQLParamData / SQLPutData
+	 *    -> cancel the statement
+	 *
+	 * 2. Running a query asynchronously. (asynchronous mode is not supported
+	 *    by psqlODBC)
+	 *
+	 * 3. Busy running a function in another thread.
+	 *    -> Send a query cancel request to the server
+	 *
+	 * 4. Not doing anything.
+	 *    -> in ODBC version 2.0, same as SQLFreeStmt(SQL_CLOSE). In version
+	 *       3.5, it has no effect.
+	 *
+	 * XXX: Checking for these conditions is racy. For example, we might
+	 * see that the statement is waiting for SQLParamdata/SQLPutData, but
+	 * before we acquire the lock on the statement, another thread has
+	 * supplied the data and started executing. In that case, we'll block
+	 * on the lock until the execution finishes.
 	 */
-	if (estmt->data_at_exec < 0)
+	if (estmt->data_at_exec >= 0)
 	{
+		/* Waiting for more data from SQLParamData/SQLPutData. Cancel that. */
 		/*
-		 * Tell the Backend that we're cancelling this request
-		 */
-		if (estmt->status == STMT_EXECUTING)
-		{
-			if (!CC_send_cancel_request(conn))
-			{
-				ret = SQL_ERROR;
-			}
-			goto cleanup;
-		}
-		/*
-		 * MAJOR HACK for Windows to reset the driver manager's cursor
-		 * state: Because of what seems like a bug in the Odbc driver
-		 * manager, SQLCancel does not act like a SQLFreeStmt(CLOSE), as
-		 * many applications depend on this behavior.  So, this brute
-		 * force method calls the driver manager's function on behalf of
-		 * the application.
+		 * Note, any previous data-at-exec buffers will be freed in the
+		 * recycle
 		 */
+		/* if they call SQLExecDirect or SQLExecute again. */
 
-		if (conn->driver_version < 0x0350)
-		{
-#ifdef WIN32
-		ConnInfo   *ci = &(conn->connInfo);
-
-		if (ci->drivers.cancel_as_freestmt)
-		{
-	typedef SQLRETURN (SQL_API *SQLAPIPROC)();
-			HMODULE		hmodule;
-			SQLAPIPROC	addr;
-
-			hmodule = GetModuleHandle("ODBC32");
-			addr = (SQLAPIPROC) GetProcAddress(hmodule, "SQLFreeStmt");
-			ret = addr((char *) (stmt->phstmt) - 96, SQL_CLOSE);
-		}
-		else
-#endif /* WIN32 */
-		{
-			ENTER_STMT_CS(stmt);
-			entered_cs = TRUE;
-			SC_clear_error(hstmt);
-			ret = PGAPI_FreeStmt(hstmt, SQL_CLOSE);
-		}
-
-		mylog("PGAPI_Cancel:  PGAPI_FreeStmt returned %d\n", ret);
-		}
-		goto cleanup;
-	}
-
-	/* In the middle of SQLParamData/SQLPutData, so cancel that. */
-	/*
-	 * Note, any previous data-at-exec buffers will be freed in the
-	 * recycle
-	 */
-	/* if they call SQLExecDirect or SQLExecute again. */
-
-	ENTER_STMT_CS(stmt);
-	entered_cs = TRUE;
-	SC_clear_error(stmt);
-	estmt->data_at_exec = -1;
-	estmt->current_exec_param = -1;
-	estmt->put_data = FALSE;
-	cancelNeedDataState(estmt);
-
-cleanup:
-#undef	return
-	if (entered_cs)
-	{
+		ENTER_STMT_CS(stmt);
+		SC_clear_error(stmt);
+		estmt->data_at_exec = -1;
+		estmt->current_exec_param = -1;
+		estmt->put_data = FALSE;
+		cancelNeedDataState(estmt);
 		if (stmt->internal)
 			ret = DiscardStatementSvp(stmt, ret, FALSE);
 		LEAVE_STMT_CS(stmt);
+		return ret;
+	}
+	else if (estmt->status == STMT_EXECUTING)
+	{
+		/*
+		 * Busy executing in a different thread. Send a cancel request to
+		 * the server.
+		 */
+		if (!CC_send_cancel_request(conn))
+			return SQL_ERROR;
+		else
+			return SQL_SUCCESS;
+	}
+	else
+	{
+		/*
+		 * The statement is not executing, and it's not waiting for params
+		 * either. Looks like it's not doing anything.
+		 */
+		if (conn->driver_version >= 0x0350)
+			return SQL_SUCCESS;
+		else
+		{
+			/*
+			 * MAJOR HACK for Windows to reset the driver manager's cursor
+			 * state: Because of what seems like a bug in the Odbc driver
+			 * manager, SQLCancel does not act like a SQLFreeStmt(CLOSE), as
+			 * many applications depend on this behavior.  So, this brute
+			 * force method calls the driver manager's function on behalf of
+			 * the application.
+			 */
+#ifdef WIN32
+			if (conn->connInfo.drivers.cancel_as_freestmt)
+			{
+	typedef SQLRETURN (SQL_API *SQLAPIPROC)();
+				HMODULE		hmodule;
+				SQLAPIPROC	addr;
+
+				hmodule = GetModuleHandle("ODBC32");
+				addr = (SQLAPIPROC) GetProcAddress(hmodule, "SQLFreeStmt");
+				ret = addr((char *) (stmt->phstmt) - 96, SQL_CLOSE);
+			}
+			else
+#endif /* WIN32 */
+			{
+				ENTER_STMT_CS(stmt);
+				SC_clear_error(hstmt);
+				ret = PGAPI_FreeStmt(hstmt, SQL_CLOSE);
+				if (stmt->internal)
+					ret = DiscardStatementSvp(stmt, ret, FALSE);
+				LEAVE_STMT_CS(stmt);
+			}
+			mylog("PGAPI_Cancel:  PGAPI_FreeStmt returned %d\n", ret);
+			return ret;
+		}
 	}
-	return ret;
 }
 
 
@@ -1418,12 +1411,6 @@ PGAPI_ParamData(HSTMT hstmt,
 
 	mylog("%s: entering...\n", func);
 
-	if (!stmt)
-	{
-		SC_log_error(func, "", NULL);
-		retval = SQL_INVALID_HANDLE;
-		goto cleanup;
-	}
 	conn = SC_get_conn(stmt);
 
 	estmt = stmt->execute_delegate ? stmt->execute_delegate : stmt;
@@ -1576,12 +1563,6 @@ PGAPI_PutData(HSTMT hstmt,
 	mylog("%s: entering...\n", func);
 
 #define	return	DONT_CALL_RETURN_FROM_HERE???
-	if (!stmt)
-	{
-		SC_log_error(func, "", NULL);
-		retval = SQL_INVALID_HANDLE;
-		goto cleanup;
-	}
 	if (SC_AcceptedCancelRequest(stmt))
 	{
 		SC_set_error(stmt, STMT_OPERATION_CANCELLED, "Cancel the statement, sorry.", func);
