@@ -34,6 +34,8 @@ BOOL	connect_with_param_available(void);
 RETCODE	CALL_EnlistInDtc(ConnectionClass *conn, void * pTra, int method);
 RETCODE	CALL_DtcOnDisconnect(ConnectionClass *);
 RETCODE	CALL_IsolateDtcConn(ConnectionClass *, BOOL);
+void	*CALL_GetTransactionObject(HRESULT *);
+void	CALL_ReleaseTransactionObject(void *);
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 /* void	UnloadDelayLoadedDLLs(BOOL); */
 void	CleanupDelayLoadedDLLs(void);
