@@ -34,7 +34,9 @@
 #include <sql.h>
 #define	_MYLOG_FUNCS_IMPORT_
 #include "mylog.h"
+#define	_PGENLIST_FUNCS_IMPLEMENT_
 #include "pgenlist.h"
+#include "xalibname.h"
 
 #ifdef WIN32
 #ifndef snprintf
@@ -57,8 +59,7 @@ DllMain(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
         switch (ul_reason_for_call)
         {
                 case DLL_PROCESS_ATTACH:
-                        s_hModule = (HINSTANCE) hInst;  /* Save for dialog boxes
- */
+                        s_hModule = (HINSTANCE) hInst;  /* Save for dialog boxes */
 			break;
 	}
 	return TRUE;
@@ -337,8 +338,10 @@ ULONG	STDMETHODCALLTYPE IAsyncPG::Release(void)
 		SLOCK_ACQUIRE();
 		if (refcnt <=0)
 		{
+			const int refcnt_copy = refcnt;
 			mylog("delete %p\n", this);
 			delete this;
+			return refcnt_copy;
 		}
 		else
 		{
@@ -644,7 +647,6 @@ HRESULT IAsyncPG::RequestExec(DWORD type, HRESULT res)
 				SetDone(res);
 				bReleaseEnlist = true;
 			}
-			PgDtc_set_property(dtcconn, prepareRequested, (void *) 0);
 			ret = enlist->PrepareRequestDone(res, NULL, NULL);
 			SetPrepareResult(res);
 			break;
@@ -998,16 +1000,82 @@ EXTERN_C static unsigned WINAPI DtcRequestExec(LPVOID para)
 
 CSTR	regKey = "SOFTWARE\\Microsoft\\MSDTC\\XADLL";
 
-RETCODE static EnlistInDtc_1pipe(void *conn, ITransaction *pTra, ITransactionDispenser *pDtc)
+static int regkeyCheck(const char *xalibname, const char *xalibpath)
+{
+	int	retcode = 0;
+	LONG	ret;
+	HKEY	sKey;
+	DWORD	rSize;
+
+	ret = ::RegOpenKeyEx(HKEY_LOCAL_MACHINE, regKey, 0, KEY_QUERY_VALUE | KEY_SET_VALUE | KEY_WOW64_64KEY, &sKey);
+	switch (ret)
+	{
+		case ERROR_SUCCESS:
+			break;
+		case ERROR_FILE_NOT_FOUND:
+			ret = ::RegCreateKeyEx(HKEY_LOCAL_MACHINE, regKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &sKey, NULL);
+			mylog("%s:CreateKeyEx ret=%d\n", __FUNCTION__, ret);
+			break;
+		default:
+			mylog("%s:OpenKeyEx ret=%d\n", __FUNCTION__, ret);
+	}
+	if (ERROR_SUCCESS != ret)
+		return -1;
+	else
+	{
+		char keyval[1024];
+
+		rSize = sizeof(keyval);
+		switch (ret = ::RegQueryValueEx(sKey, xalibname, NULL, NULL, (LPBYTE) keyval, &rSize))
+		{
+			case ERROR_SUCCESS:
+				if (rSize > 0)
+				{
+					if (0 == _stricmp(keyval, xalibpath))
+						break;
+					mylog("%s:XADLL value %s is different from %s\n", __FUNCTION__, keyval, xalibpath);
+					if (IsWow64())
+					{
+						mylog("%s:avoid RegSetValue operation from wow64 process\n", __FUNCTION__);
+						break;
+					}
+				}
+			case ERROR_FILE_NOT_FOUND:
+				mylog("%s:Setting value %s\n", __FUNCTION__, xalibpath);
+				ret = ::RegSetValueEx(sKey, xalibname, 0, REG_SZ, (CONST BYTE *) xalibpath, (DWORD) strlen(xalibpath) + 1);
+				if (ERROR_SUCCESS == ret)
+					retcode = 1;
+				else
+				{
+					retcode = -1;
+					mylog("%s:SetValuEx ret=%d\n", __FUNCTION__, ret);
+				}
+				break;
+			default:
+				retcode = -1;
+				mylog("%s:QueryValuEx ret=%d\n", __FUNCTION__, ret);
+				break;
+		}
+		::RegCloseKey(sKey);
+	}
+	return retcode;
+}
+
+RETCODE static EnlistInDtc_1pipe(void *conn, ITransaction *pTra, ITransactionDispenser *pDtc, int method)
 {
 	CSTR	func = "EnlistInDtc_1pipe";
 	static	IDtcToXaHelperSinglePipe	*pHelper = NULL;
 	ITransactionResourceAsync		*pRes = NULL;
 	IAsyncPG				*asdum;
 	HRESULT	res;
-	bool	retry, errset;
 	DWORD	dwRMCookie;
 	XID	xid;
+	const char *xalibname = GetXaLibName();
+	const char *xalibpath = GetXaLibPath();
+
+	int	recovLvl;
+	char	errmsg[256];
+	char	reason[128];
 
 	if (!pHelper)
 	{
@@ -1026,55 +1094,94 @@ RETCODE static EnlistInDtc_1pipe(void *conn, ITransaction *pTra, ITransactionDis
 		return SQL_ERROR;
 	}
 
-/*mylog("dllname=%s dsn=%s\n", GetXaLibName(), conn->connInfo.dsn); res = 0;*/
-	retry = false;
-	errset = false;
+	recovLvl = PgDtc_is_recovery_available(conn, reason, sizeof(reason));
+	switch (method)
+	{
+		case DTC_CHECK_BEFORE_LINK:
+			if (0 == recovLvl)
+			{
+				snprintf(errmsg, sizeof(errmsg), "%s is unavailable in distributed transactions", reason);
+				PgDtc_set_error(conn, errmsg, func);
+				return SQL_ERROR;
+			}
+	}
+/*mylog("dllname=%s dsn=%s\n", xalibname, conn->connInfo.dsn); res = 0;*/
 	char	dtcname[1024];
 	PgDtc_create_connect_string(conn, dtcname, sizeof(dtcname));
-	do {
-		res = pHelper->XARMCreate(dtcname, (char *) GetXaLibName(), &dwRMCookie);
-		if (S_OK == res)
-			break;
-		mylog("XARMCreate error code=%x\n", res);
-		if (XACT_E_XA_TX_DISABLED == res)
-		{
-			PgDtc_set_error(conn, "XARMcreate error:Please enable XA transaction in MSDTC security configuration", func);
-			errset = true;
-		}
-		else if (!retry)
-		{
-			LONG	ret;
-			HKEY	sKey;
-			DWORD	rSize;
 
-			ret = ::RegOpenKeyEx(HKEY_LOCAL_MACHINE, regKey, 0, KEY_QUERY_VALUE | KEY_SET_VALUE, &sKey);
-			if (ERROR_SUCCESS != ret)
-				ret = ::RegCreateKeyEx(HKEY_LOCAL_MACHINE, regKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &sKey, NULL);
-			if (ERROR_SUCCESS == ret)
-			{
-				switch (ret = ::RegQueryValueEx(sKey, "XADLL", NULL, NULL, NULL, &rSize))
+	bool	confirmedRegkey = false, confirmingLink = false, xarmerr = false;
+	char	error_header[64];
+	while (true)
+	{
+		res = pHelper->XARMCreate(dtcname, (char *) xalibname, &dwRMCookie);
+
+		mylog("XARMcreate error code=%x (%d %d)\n", res, confirmedRegkey, confirmingLink);
+		xarmerr = true;
+		if (!confirmingLink)
+			snprintf(error_header, sizeof(error_header), "XARMcreate error code=%x", res);
+		switch (res)
+		{
+			case S_OK:
+				if (confirmingLink)
 				{
-					case ERROR_SUCCESS:
-						if (rSize > 0)
+					switch (recovLvl)
+					{
+						case 0:
+							snprintf(errmsg, sizeof(errmsg), "%s:%s is currently unavailable in distributed transactions", error_header, reason);
 							break;
-					default:
-						ret = ::RegSetValueEx(sKey, GetXaLibName(), 0, REG_SZ,
-											  (CONST BYTE *) GetXaLibPath(), (DWORD) strlen(GetXaLibPath()) + 1);
-						if (ERROR_SUCCESS == ret)
-						{
-							retry = true;
-							continue; // retry
-						}
-						PgDtc_set_error(conn, "XARMCreate error:Please register HKLM\\SOFTWARE\\Microsoft\\MSDTC\\XADLL", func);
-						break;
+						case -1:
+							snprintf(errmsg, sizeof(errmsg), "%s:Possibly you connect to the database whose authentication method is %s or ident", error_header, reason);
+							break;
+						case 1:
+							snprintf(errmsg, sizeof(errmsg), "%s:Are you trying to connect to the database whose authentication method is ident?", error_header, reason);
+							break;
+					}
 				}
-				::RegCloseKey(sKey);
-			}
+				else
+					xarmerr = false;
+				break;
+			case XACT_E_XA_TX_DISABLED:
+				snprintf(errmsg, sizeof(errmsg), "%s:Please enable XA transaction in MSDTC security configuration", error_header);
+				break;
+			case XACT_E_TMNOTAVAILABLE:
+				snprintf(errmsg, sizeof(errmsg), "%s:Please start Distributed Transaction Coordinator service", error_header);
+				break;
+			case E_FAIL:
+				if (!confirmedRegkey)
+				{
+					int retcode = regkeyCheck(xalibname, xalibpath);
+					confirmedRegkey = true;
+					if (retcode > 0)
+						continue;
+				}
+				switch (method)
+				{
+					case DTC_CHECK_RM_CONNECTION:
+						if (!confirmingLink)
+						{
+							confirmingLink = true;
+							strcat(dtcname, ";" KEYWORD_DTC_CHECK "=0");
+							continue;
+						}
+					default:
+						snprintf(errmsg, sizeof(errmsg), "%s:Failed to link with DTC service. Please look at the log of Event Viewer etc.", error_header);
+				}
+				break;
+			case XACT_E_CONNECTION_DOWN:
+				snprintf(errmsg, sizeof(errmsg), "%s:Lost connection with DTC transaction manager\nMSDTC has some trouble?", error_header);
+				break;
+			default:
+				snprintf(errmsg, sizeof(errmsg), "%s\n", error_header);
+				break;
 		}
-		if (!errset)
-			PgDtc_set_error(conn, "MSDTC XARMCreate error", func);
+		break;
+	}
+	if (xarmerr)
+	{
+		PgDtc_set_error(conn, errmsg, func);
 		return SQL_ERROR;
-	} while (1);
+	}
+
 	res = pHelper->ConvertTridToXID((DWORD *) pTra, dwRMCookie, &xid);
 	if (res != S_OK)
 	{
@@ -1133,9 +1240,61 @@ IsolateDtcConn(void *conn, BOOL continueConnection)
 }
 
 
-EXTERN_C RETCODE EnlistInDtc(void *conn, void *pTra, int method)
+static ITransactionDispenser *getITransactionDispenser(DWORD grfOptions, HRESULT *hres)
 {
 	static	ITransactionDispenser	*pDtc = NULL;
+	HRESULT	res = S_OK;
+
+	if (!pDtc)
+	{
+		res = DtcGetTransactionManagerEx(NULL, NULL, IID_ITransactionDispenser,
+			
+			grfOptions, NULL, (void **) &pDtc);
+		if (FAILED(res))
+		{
+			forcelog("DtcGetTransactionManager error %x\n", res);
+			pDtc = NULL;
+		}
+	}
+	if (hres) 
+		*hres = res;
+
+	return pDtc;
+}
+
+EXTERN_C void	*GetTransactionObject(HRESULT *hres)
+{
+	
+	ITransaction	*pTra = NULL;
+	ITransactionDispenser	*pDtc = NULL;
+
+	if (pDtc = getITransactionDispenser(OLE_TM_FLAG_NONE, hres), NULL == pDtc)
+		return pTra;
+	HRESULT res = pDtc->BeginTransaction(NULL, ISOLATIONLEVEL_READCOMMITTED,
+		0, NULL, &pTra);
+	switch (res)
+	{
+		case S_OK:
+			break;
+		default:
+			pTra = NULL;
+	}
+	if (hres)
+		*hres = res;
+	return pTra; 
+}
+
+EXTERN_C void	ReleaseTransactionObject(void *pObj)
+{
+	ITransaction	*pTra = (ITransaction *) pObj;
+
+	if (!pTra)	return;
+	pTra->Release();
+}
+
+EXTERN_C RETCODE EnlistInDtc(void *conn, void *pTra, int method)
+{
+	ITransactionDispenser	*pDtc = NULL;
 	RETCODE	ret;
 
 	if (!pTra)
@@ -1148,19 +1307,16 @@ EXTERN_C RETCODE EnlistInDtc(void *conn, void *pTra, int method)
 	{
 		PgDtc_one_phase_operation(conn, SHUTDOWN_LOCAL_TRANSACTION);
 	}
+	HRESULT	hres;
+	pDtc = getITransactionDispenser(OLE_TM_FLAG_NODEMANDSTART, &hres);
 	if (!pDtc)
 	{
-		HRESULT	res;
-
-		res = DtcGetTransactionManager(NULL, NULL, IID_ITransactionDispenser,
-			0, 0, NULL,  (void **) &pDtc);
-		if (res != S_OK || !pDtc)
-		{
-			forcelog("TransactionManager get error %d\n", res);
-			pDtc = NULL;
-		}
+		char	errmsg[128];
+		snprintf(errmsg, sizeof(errmsg), "enlistment error:DtcGetTransactionManager error code=%x", hres);
+		PgDtc_set_error(conn, errmsg, __FUNCTION__);
+		return SQL_ERROR;
 	}
-	ret = EnlistInDtc_1pipe(conn, (ITransaction *) pTra, pDtc);
+	ret = EnlistInDtc_1pipe(conn, (ITransaction *) pTra, pDtc, method);
 	if (SQL_SUCCEEDED(ret))
 		PgDtc_set_property(conn, enlisted, (void *) 1);
 	return ret;
