@@ -532,9 +532,6 @@ pgtype_attr_to_concise_type(const ConnectionClass *conn, OID type, int atttypmod
 	switch (type)
 	{
 		case PG_TYPE_CHAR:
-		case PG_TYPE_CHAR2:
-		case PG_TYPE_CHAR4:
-		case PG_TYPE_CHAR8:
 			return ALLOW_WCHAR(conn) ? SQL_WCHAR : SQL_CHAR;
 		case PG_TYPE_NAME:
 		case PG_TYPE_REFCURSOR:
@@ -828,12 +825,6 @@ pgtype_attr_to_name(const ConnectionClass *conn, OID type, int atttypmod, BOOL a
 	{
 		case PG_TYPE_CHAR:
 			return "char";
-		case PG_TYPE_CHAR2:
-			return "char2";
-		case PG_TYPE_CHAR4:
-			return "char4";
-		case PG_TYPE_CHAR8:
-			return "char8";
 		case PG_TYPE_INT8:
 			return auto_increment ? "bigserial" : "int8";
 		case PG_TYPE_NUMERIC:
@@ -928,12 +919,6 @@ pgtype_attr_column_size(const ConnectionClass *conn, OID type, int atttypmod, in
 	{
 		case PG_TYPE_CHAR:
 			return 1;
-		case PG_TYPE_CHAR2:
-			return 2;
-		case PG_TYPE_CHAR4:
-			return 4;
-		case PG_TYPE_CHAR8:
-			return 8;
 
 		case PG_TYPE_NAME:
 		case PG_TYPE_REFCURSOR:
@@ -981,12 +966,13 @@ pgtype_attr_column_size(const ConnectionClass *conn, OID type, int atttypmod, in
 		case PG_TYPE_NUMERIC:
 			return getNumericColumnSizeX(conn, type, atttypmod, adtsize_or_longest, handle_unknown_size_as);
 
-		case PG_TYPE_FLOAT4:
 		case PG_TYPE_MONEY:
-			return 7;
+			return 10;
+		case PG_TYPE_FLOAT4:
+			return PG_REAL_DIGITS;
 
 		case PG_TYPE_FLOAT8:
-			return 15;
+			return PG_DOUBLE_DIGITS;
 
 		case PG_TYPE_DATE:
 			return 10;
@@ -1077,11 +1063,11 @@ pgtype_attr_display_size(const ConnectionClass *conn, OID type, int atttypmod, i
 		case PG_TYPE_MONEY:
 			return 15;			/* ($9,999,999.99) */
 
-		case PG_TYPE_FLOAT4:
-			return 13;
+		case PG_TYPE_FLOAT4:	/* a sign, PG_REAL_DIGITS digits, a decimal point, the letter E, a sign, and 2 digits */
+			return (1 + PG_REAL_DIGITS + 1 + 1 + 3);
 
-		case PG_TYPE_FLOAT8:
-			return 22;
+		case PG_TYPE_FLOAT8:	/* a sign, PG_DOUBLE_DIGITS digits, a decimal point, the letter E, a sign, and 3 digits */
+			return (1 + PG_DOUBLE_DIGITS + 1 + 1 + 1 + 3);
 
 		case PG_TYPE_MACADDR:
 			return 17;
@@ -2071,10 +2057,6 @@ pgtype_case_sensitive(const ConnectionClass *conn, OID type)
 	{
 		case PG_TYPE_CHAR:
 
-		case PG_TYPE_CHAR2:
-		case PG_TYPE_CHAR4:
-		case PG_TYPE_CHAR8:
-
 		case PG_TYPE_VARCHAR:
 		case PG_TYPE_BPCHAR:
 		case PG_TYPE_TEXT:
@@ -2107,9 +2089,6 @@ pgtype_searchable(const ConnectionClass *conn, OID type)
 	switch (type)
 	{
 		case PG_TYPE_CHAR:
-		case PG_TYPE_CHAR2:
-		case PG_TYPE_CHAR4:
-		case PG_TYPE_CHAR8:
 
 		case PG_TYPE_VARCHAR:
 		case PG_TYPE_BPCHAR:
@@ -2372,6 +2351,11 @@ ctype_length(SQLSMALLINT ctype)
 		case SQL_C_INTERVAL_HOUR_TO_SECOND:
 		case SQL_C_INTERVAL_MINUTE_TO_SECOND:
 			return sizeof(SQL_INTERVAL_STRUCT);
+		case SQL_C_NUMERIC:
+			return sizeof(SQL_NUMERIC_STRUCT);
+		case SQL_C_SBIGINT:
+		case SQL_C_UBIGINT:
+			return sizeof(SQLBIGINT);
 #endif /* ODBCVER */
 
 		case SQL_C_BINARY:
