@@ -35,31 +35,16 @@
 #define PG_TYPE_XID				28
 #define PG_TYPE_CID				29
 #define PG_TYPE_OIDVECTOR		30
-#define PG_TYPE_SET				32
 #define PG_TYPE_XML			142
 #define PG_TYPE_XMLARRAY		143
-#define PG_TYPE_CHAR2			409
-#define PG_TYPE_CHAR4			410
-#define PG_TYPE_CHAR8			411
-#define PG_TYPE_POINT			600
-#define PG_TYPE_LSEG			601
-#define PG_TYPE_PATH			602
-#define PG_TYPE_BOX				603
-#define PG_TYPE_POLYGON			604
-#define PG_TYPE_FILENAME		605
 #define PG_TYPE_CIDR			650
 #define PG_TYPE_FLOAT4			700
 #define PG_TYPE_FLOAT8			701
 #define PG_TYPE_ABSTIME			702
-#define PG_TYPE_RELTIME			703
-#define PG_TYPE_TINTERVAL		704
 #define PG_TYPE_UNKNOWN			705
 #define PG_TYPE_MONEY			790
-#define PG_TYPE_OIDINT2			810
 #define PG_TYPE_MACADDR			829
 #define PG_TYPE_INET			869
-#define PG_TYPE_OIDINT4			910
-#define PG_TYPE_OIDNAME			911
 #define PG_TYPE_TEXTARRAY		1009
 #define PG_TYPE_BPCHARARRAY		1014
 #define PG_TYPE_VARCHARARRAY		1015
