@@ -196,7 +196,6 @@ QR_Constructor(void)
 		rv->notice = NULL;
 		rv->conn = NULL;
 		rv->next = NULL;
-		rv->pstatus = 0;
 		rv->count_backend_allocated = 0;
 		rv->count_keyset_allocated = 0;
 		rv->num_total_read = 0;
@@ -319,6 +318,25 @@ QR_close_result(QResultClass *self, BOOL destroy)
 	mylog("QResult: exit close_result\n");
 }
 
+void
+QR_reset_for_re_execute(QResultClass *self)
+{
+	mylog("QResult: enter %s for %x\n", __FUNCTION__, self);
+	if (!self)	return;
+	QR_close_result(self, FALSE);
+	/* reset flags etc */
+	self->flags = 0;
+	QR_set_rowstart_in_cache(self, -1);
+	self->recent_processed_row_count = -1;
+	/* clear error info etc */
+	self->rstatus = PORES_EMPTY_QUERY;
+	self->aborted = FALSE;
+	self->sqlstate[0] = '\0';
+	self->messageref = NULL;
+
+	mylog("QResult: exit %s\n", __FUNCTION__);
+}
+
 void
 QR_Destructor(QResultClass *self)
 {
@@ -464,6 +482,7 @@ QR_free_memory(QResultClass *self)
 		self->count_backend_allocated = 0;
 		self->backend_tuples = NULL;
 		self->dataFilled = FALSE;
+		self->tupleField = NULL;
 	}
 	if (self->keyset)
 	{
@@ -681,7 +700,6 @@ void
 QR_on_close_cursor(QResultClass *self)
 {
 	QR_set_cursor(self, NULL);
-	QR_set_has_valid_base(self);
 }
 
 /*
@@ -927,7 +945,7 @@ inolog("in total_read=%d cursT=%d currT=%d ad=%d total=%d rowsetSize=%d\n", self
 	if (0 != self->move_offset)
 	{
 		char		movecmd[256];
-		QResultClass	*mres;
+		QResultClass	*mres = NULL;
 		SQLULEN		movement, moved;
 
 		movement = self->move_offset;
@@ -993,7 +1011,7 @@ inolog("moved=%d ? " FORMAT_ULEN "\n", moved, movement);
 				if (QR_is_moving_from_the_last(self))  /* in case of FETCH LAST */
 				{
 					SQLULEN	bmovement, mback;
-					SQLLEN	rowset_start = self->cursTuple + 1, back_offset = self->move_offset, backpt;
+					SQLLEN	rowset_start = self->cursTuple + 1, back_offset, backpt;
 inolog("FETCH LAST case\n");
 					if (getNthValid(self, QR_get_num_total_tuples(self) - 1, SQL_FETCH_PRIOR, self->move_offset, &backpt) < 0)
 					{
@@ -1001,6 +1019,7 @@ inolog("FETCH LAST case\n");
 						self->tupleField = NULL;
 						SC_set_rowset_start(stmt, -1, TRUE);
 						stmt->currTuple = -1;
+						QR_Destructor(mres);
 						RETURN(-1)
 					}
 					back_offset = QR_get_num_total_tuples(self) - backpt;
@@ -1686,6 +1705,15 @@ else
 
 			if (field_lf >= effective_cols)
 			{
+				if (NULL == this_keyset)
+				{
+					char	emsg[128];
+
+					QR_set_rstatus(self, PORES_INTERNAL_ERROR);
+					snprintf(emsg, sizeof(emsg), "Internal Error -- this_keyset == NULL ci_num_fields=%d effective_cols=%d", ci_num_fields, effective_cols);
+					QR_set_message(self, emsg);
+					return FALSE;
+				}
 				if (field_lf == effective_cols)
 					sscanf(buffer, "(%u,%hu)",
 						&this_keyset->blocknum, &this_keyset->offset);
