@@ -215,13 +215,13 @@ PGAPI_AllocStmt(HDBC hdbc,
 	if (0 != (PODBC_INHERIT_CONNECT_OPTIONS & flag))
 	{
 		stmt->options = stmt->options_orig = conn->stmtOptions;
-		stmt->ardi.ardopts = conn->ardOptions;
+		stmt->ardi.ardf = conn->ardOptions;
 	}
 	else
 	{
 		InitializeStatementOptions(&stmt->options_orig);
 		stmt->options = stmt->options_orig;
-		InitializeARDFields(&stmt->ardi.ardopts);
+		InitializeARDFields(&stmt->ardi.ardf);
 	}
 	ardopts = SC_get_ARDF(stmt);
 	ARD_AllocBookmark(ardopts);
@@ -317,6 +317,7 @@ PGAPI_FreeStmt(HSTMT hstmt,
 		{
 			return SQL_ERROR;
 		}
+		SC_set_Curres(stmt, NULL);
 	}
 	else if (fOption == SQL_RESET_PARAMS)
 		SC_free_params(stmt, STMT_FREE_PARAMS_ALL);
@@ -805,6 +806,27 @@ SC_initialize_and_recycle(StatementClass *self)
 	return SQL_SUCCESS;
 }
 
+void
+SC_reset_result_for_rerun(StatementClass *self)
+{
+	QResultClass	*res;
+	ColumnInfoClass	*flds;
+
+	if (!self)	return;
+	if (res = SC_get_Result(self), NULL == res)
+		return;
+	flds = QR_get_fields(res);
+	if (NULL == flds ||
+	    0 == CI_get_num_fields(flds))
+		SC_set_Result(self, NULL);
+	else
+	{
+		QR_reset_for_re_execute(res);
+		self->curr_param_result = 1;
+		SC_set_Curres(self, NULL);
+	}
+}
+
 /*
  *	Called from SQLPrepare if STMT_PREMATURE, or
  *	from SQLExecute if STMT_FINISHED, or
@@ -880,11 +902,10 @@ inolog("SC_clear_parse_status\n");
 		{
 			case PREPARED_PERMANENTLY:
 			case PREPARED_TEMPORARILY:
-				QR_close_result(res, FALSE);
+				SC_reset_result_for_rerun(self);
 				break;
 			default:
-				QR_Destructor(res);
-				SC_init_Result(self);
+				SC_set_Result(self, NULL);
 				break;
 		}
 	}
@@ -2260,6 +2281,8 @@ inolog("res->next=%p\n", tres);
 			last->next = res;
 		self->curr_param_result = 1;
 	}
+	if (NULL == SC_get_Curres(self))
+		SC_set_Curres(self, SC_get_Result(self));
 
 	ipdopts = SC_get_IPDF(self);
 	has_out_para = FALSE;
@@ -2533,7 +2556,8 @@ ReflectColumnsInfo(StatementClass *self, QResultClass *res)
 	if (res->num_fields > 0)
 		return FALSE;
 	pres = SC_get_Result(self);
-	if (pres != res &&
+	if (pres != NULL &&
+	    pres != res &&
 	    pres->num_fields > 0)
 	{
 		QR_set_fields(res, QR_get_fields(pres));
@@ -2569,7 +2593,6 @@ QResultClass *SendSyncAndReceive(StatementClass *stmt, QResultClass *res, const
 {
 	CSTR func = "SendSyncAndReceive";
 	ConnectionClass	*conn = SC_get_conn(stmt);
-	SocketClass	*sock = conn->sock;
 	char		id;
 	Int4		response_length;
 	UInt4		oid;
@@ -2584,26 +2607,26 @@ QResultClass *SendSyncAndReceive(StatementClass *stmt, QResultClass *res, const
 	if (!RequestStart(stmt, conn, func))
 		return NULL;
 
-	SOCK_put_char(sock, 'S');	/* Sync command */
-	SOCK_put_int(sock, 4, 4);	/* length */
-	SOCK_flush_output(sock);
+	SOCK_put_char(conn->sock, 'S');	/* Sync command */
+	SOCK_put_int(conn->sock, 4, 4);	/* length */
+	SOCK_flush_output(conn->sock);
 
 	if (!res)
 		newres = res = QR_Constructor();
 	for (;!loopend;)
 	{
-		id = SOCK_get_id(sock);
-		if ((SOCK_get_errcode(sock) != 0) || (id == EOF))
+		id = SOCK_get_id(conn->sock);
+		if ((SOCK_get_errcode(conn->sock) != 0) || (id == EOF))
 			break;
 inolog("desc id=%c", id);
-		response_length = SOCK_get_response_length(sock);
-		if (0 != SOCK_get_errcode(sock))
+		response_length = SOCK_get_response_length(conn->sock);
+		if (0 != SOCK_get_errcode(conn->sock))
 			break;
 inolog(" response_length=%d\n", response_length);
 		switch (id)
 		{
 			case 'C':
-				SOCK_get_string(sock, msgbuffer, sizeof(msgbuffer));
+				SOCK_get_string(conn->sock, msgbuffer, sizeof(msgbuffer));
 				mylog("command response=%s\n", msgbuffer);
 				QR_set_command(res, msgbuffer);
 				if (QR_is_fetching_tuples(res))
@@ -2627,6 +2650,8 @@ inolog(" response_length=%d\n", response_length);
 							res->recent_processed_row_count = ret1;
 					}
 				}
+				else if (QR_command_successful(res))
+					QR_set_rstatus(res, PORES_COMMAND_OK);
 				break;
 			case 'E': /* ErrorMessage */
 				handle_error_message(conn, msgbuffer, sizeof(msgbuffer), res->sqlstate, comment, res);
@@ -2652,7 +2677,7 @@ inolog(" response_length=%d\n", response_length);
 				EatReadyForQuery(conn);
 				break;
 			case 't': /* ParameterDesription */
-				num_p = SOCK_get_int(sock, 2);
+				num_p = SOCK_get_int(conn->sock, 2);
 inolog("num_params=%d info=%d\n", stmt->num_params, num_p);
 				num_discard_params = 0;
 				if (stmt->discard_output_params)
@@ -2677,7 +2702,7 @@ inolog("num_params=%d info=%d\n", stmt->num_params, num_p);
 							i++;
 							continue;
 						}
-						oid = SOCK_get_int(sock, 4);
+						oid = SOCK_get_int(conn->sock, 4);
 						PIC_set_pgtype(ipdopts->parameters[pidx], oid);
 					}
 				}
@@ -2686,7 +2711,7 @@ inolog("num_params=%d info=%d\n", stmt->num_params, num_p);
 					for (i = 0, pidx = stmt->proc_return; i < num_p; i++, pidx++)
 					{
 						paramType = ipdopts->parameters[pidx].paramType;
-						oid = SOCK_get_int(sock, 4);
+						oid = SOCK_get_int(conn->sock, 4);
 						if (SQL_PARAM_OUTPUT != paramType ||
 						    PG_TYPE_VOID != oid)
 							PIC_set_pgtype(ipdopts->parameters[pidx], oid);
@@ -2704,7 +2729,7 @@ inolog("num_params=%d info=%d\n", stmt->num_params, num_p);
 						mylog("%dth parameter's position(%d) is out of bound[%d]\n", i, pidx, stmt->num_params);
 						break;
 					}
-					oid = SOCK_get_int(sock, 4);
+					oid = SOCK_get_int(conn->sock, 4);
 					paramType = ipdopts->parameters[pidx].paramType;
 					if (SQL_PARAM_OUTPUT != paramType ||
 					    PG_TYPE_VOID != oid)
@@ -2777,15 +2802,15 @@ inolog("!![%d].PGType %u->%u\n", i, PIC_get_pgtype(ipdopts->parameters[i]), CI_g
 				break;
 		}
 	}
-	if (!rcvend && 0 == SOCK_get_errcode(sock) && EOF != id)
+	if (!rcvend && 0 == SOCK_get_errcode(conn->sock) && EOF != id)
 	{
 		for (;;)
 		{
-			id = SOCK_get_id(sock);
-			if ((SOCK_get_errcode(sock) != 0) || (id == EOF))
+			id = SOCK_get_id(conn->sock);
+			if ((SOCK_get_errcode(conn->sock) != 0) || (id == EOF))
 				break;
-			response_length = SOCK_get_response_length(sock);
-			if (0 != SOCK_get_errcode(sock))
+			response_length = SOCK_get_response_length(conn->sock);
+			if (0 != SOCK_get_errcode(conn->sock))
 				break;
 			if ('Z' == id)
 			{
@@ -2795,7 +2820,7 @@ inolog("!![%d].PGType %u->%u\n", i, PIC_get_pgtype(ipdopts->parameters[i]), CI_g
 			}
 		}
 	}
-	if (0 != SOCK_get_errcode(sock) || EOF == id)
+	if (0 != SOCK_get_errcode(conn->sock) || EOF == id)
 	{
 		SC_set_error(stmt, STMT_NO_RESPONSE, "No response from the backend", func);
 
